/*
 * Copyright notice from original mutt:
 * Copyright (C) 2003 Werner Koch <wk@gnupg.org>
 * Copyright (C) 2004 g10code GmbH
 *
 * This file is part of mutt-ng, see http://www.muttng.org/.
 * It's licensed under the GNU General Public License,
 * please see the file GPL in the top level source directory.
 */

/*
   Common definitions and prototypes for the crypt functions. They are
   all defined in crypt.c and cryptglue.c
*/

#ifndef MUTT_CRYPT_H
#define MUTT_CRYPT_H

#include <lib-mime/mime.h>
#include "state.h"
#include "crypt.li"

#define ENCRYPT    (1 << 0)
#define SIGN       (1 << 1)
#define GOODSIGN   (1 << 2)
#define BADSIGN    (1 << 3)
#define PARTSIGN   (1 << 4)
#define SIGNOPAQUE (1 << 5)
#define KEYBLOCK   (1 << 6)     /* KEY too generic? */
#define INLINE     (1 << 7)

#define APPLICATION_PGP    (1 << 8)
#define APPLICATION_SMIME  (1 << 9)

#define PGP_TRADITIONAL_CHECKED (1 << 10)

#define PGPENCRYPT  (APPLICATION_PGP | ENCRYPT)
#define PGPSIGN     (APPLICATION_PGP | SIGN)
#define PGPKEY      (APPLICATION_PGP | KEYBLOCK)
#define PGPINLINE   (APPLICATION_PGP | INLINE)

#define SMIMEENCRYPT  (APPLICATION_SMIME | ENCRYPT)
#define SMIMESIGN     (APPLICATION_SMIME | SIGN)
#define SMIMEOPAQUE   (APPLICATION_SMIME | SIGNOPAQUE)

#define KEYFLAG_CANSIGN 		(1 <<  0)
#define KEYFLAG_CANENCRYPT 		(1 <<  1)
#define KEYFLAG_ISX509                  (1 <<  2)
#define KEYFLAG_SECRET			(1 <<  7)
#define KEYFLAG_EXPIRED 		(1 <<  8)
#define KEYFLAG_REVOKED 		(1 <<  9)
#define KEYFLAG_DISABLED 		(1 << 10)
#define KEYFLAG_SUBKEY 			(1 << 11)
#define KEYFLAG_CRITICAL 		(1 << 12)
#define KEYFLAG_PREFER_ENCRYPTION 	(1 << 13)
#define KEYFLAG_PREFER_SIGNING 		(1 << 14)

#define KEYFLAG_CANTUSE (KEYFLAG_DISABLED|KEYFLAG_REVOKED|KEYFLAG_EXPIRED)
#define KEYFLAG_RESTRICTIONS (KEYFLAG_CANTUSE|KEYFLAG_CRITICAL)

#define KEYFLAG_ABILITIES (KEYFLAG_CANSIGN|KEYFLAG_CANENCRYPT|KEYFLAG_PREFER_ENCRYPTION|KEYFLAG_PREFER_SIGNING)

int crypt_send_menu(HEADER *msg, int *redraw, int smime);
int mutt_protect (HEADER *, char *);
int mutt_signed_handler (BODY *, STATE *);
int mutt_parse_crypt_hdr (char *, int);
int mutt_check_traditional_pgp (HEADER *, int *);

/* Check out the type of encryption used and set the cached status
   values if there are any. */
int crypt_query (BODY * m);

/* Fixme: To be documented. */
void crypt_extract_keys_from_messages (HEADER * h);

/* Do a quick check to make sure that we can find all of the
   encryption keys if the user has requested this service. 
   Return the list of keys in KEYLIST. */
int crypt_get_keys (HEADER * msg, char **keylist);

/* Decrypt a PGP/MIME message. */
int crypt_pgp_decrypt_mime (FILE * a, FILE ** b, BODY * c, BODY ** d);

/* MIME handler for the application/pgp content-type. */
int crypt_pgp_application_pgp_handler (BODY * m, STATE * s);

/* MIME handler for an PGP/MIME encrypted message. */
int crypt_pgp_encrypted_handler (BODY * a, STATE * s);

/* Check for a traditional PGP message in body B. */
int crypt_pgp_check_traditional (FILE * fp, BODY * b, int tagged_only);

/* fixme: needs documentation */
void crypt_pgp_extract_keys_from_attachment_list (FILE * fp, int tag,
                                                  BODY * top);


/* Decrypt an S/MIME message. */
int crypt_smime_decrypt_mime (FILE * a, FILE ** b, BODY * c, BODY ** d);

/* MIME handler for the application/smime content-type. */
int crypt_smime_application_smime_handler (BODY * m, STATE * s);

/* Check that the sender matches. */
int crypt_smime_verify_sender (HEADER * h);

#endif /*MUTT_CRYPT_H */
